#! /usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *
import os

class CodeDocument(QTextDocument):
    '''
    The text model
    @arg parent: The parent should be the CodeEditor
    '''
    # Segmentation fault is produced when 
    def __init__(self, parent, url = None):
        QTextDocument.__init__(self, parent)
        self._qfile = None
        
    def load(self, url):
        
        if isinstance(url, (str, QString)):
            self._qfile = QFile(url, self)
            self._qfile.open(QIODevice.Text | QIODevice.ReadWrite)
            data = self._qfile.read(self._qfile.size())
            self.setPlainText(QString(data))
            title = self._qfile.fileName().split(os.sep).last()
            print "Setting title"
            self.setMetaInformation(QTextDocument.DocumentTitle, title)
            print "Title set"
        else:
            print "Can't handle %s" % url
            
    def path():
        doc = "Full file path"
        def fget(self):
            return self._qfile and self._qfile.fileName() or None
        return locals()
    path = property(**path())
    
    def save(self):
        QMessageBox.information(self, 'Save', 'Save')
    
    #def setMetaInformation(self, info, string):
        #QTextDocument.setMetaInformation(self, info, string)
        #if info == QTextDocument.DocumentTitle:
            #self.emit(SIGNAL('titleChanged'), string)

class InternalEditor(QTextEdit):
    # Mimic simple style
    DEFAULT_STYLE = '''
        QTextEdit {
            selection-color: #dea;
            selection-background-color: #CBE4FA;
            selection-color: #222222;  
            border-top: 1px;
            border-left: 1px solid #888;
            padding: 0px;
            margin: 0px;
        }
    '''
    DEFAULT_FONT = QFont('Monaco', 8)
    
    def __init__(self, parent, document = None):
        assert isinstance(parent, QWidget), "Parent != QWudget???"
        QTextEdit.__init__(self, parent)
        #self.setDocument(document or Document(self))
        self.setAcceptRichText(False)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.setFont(self.DEFAULT_FONT)
        self.connect(self.document(), SIGNAL('blockCountChanged(int)'), 
                        parent.linenumber_col.update_width)
        self.setStyleSheet( self.DEFAULT_STYLE)
        print "Setting custom document"
        doc = CodeDocument(self)
        self.setDocument(doc)
        
    def title():
        doc = "Document tilte"
        def fget(self):
            return self.document().metaInformation(QTextDocument.DocumentTitle) or "untitled"
        def fset(self, val):
            self.document().setMetaInformation(QTextDocument.DocumentTitle, val)
        return locals()
    title = property(**title())
    
    def paintEvent(self, e):
        # When the text changes, alos the left bars should
        self.parent().linenumber_col.update()
        self.parent().bookmark_col.update()
        self.parent().folding_col.update()
        QTextEdit.paintEvent(self, e)
        #print self.cursorRect()
    
    def keyPressEvent(self, e):
        QTextEdit.keyPressEvent(self,e)
    
    
    def setDocument(self, document):
        print "Settings document %s for %s" % (document, self)
        print document.parent() is self
        print document.toPlainText()
        QTextEdit.setDocument(self, document)
        print "Document set"
        
class LineNumberColumn(QWidget):
    
    def __init__(self, parent):
        QWidget.__init__(self, parent)
        self.__char_length = 1
        self.update_length()
        # Text color
        self._pen = QPen(QColor(0x88, 0x88, 0x88))
    
    def char_length():
        def fget(self):
            return self.__char_length
        
        def fset(self, value):
            if value != self.__char_length:
                self.__char_length = value
                self.update_length()
            
        return locals()
    char_length = property(**char_length())
    
    def update_length(self):
        #try:
            #fm = self.parent().editor.fontMetrics()
        #except:
            #fm = self.fontMetrics()
        fm = self.fontMetrics()
        self.setMinimumWidth(fm.width('w') * self.char_length)
        # TODO: Check this
        #print self.minimumWidth()
    
    def update_width(self, blocks):
        length = 0
        while blocks > 0:
            blocks /= 10
            length += 1
        self.char_length = length
        
        
    def paintEvent(self, e):
        p = QPainter(self)
        p.setPen(self._pen)
        editor = self.parent().editor
        
        layout = editor.document().documentLayout()
        contentsY = editor.verticalScrollBar().value()
        pageBottom = contentsY + editor.viewport().height()
        fm = editor.fontMetrics()
        ascent = editor.fontMetrics().ascent() + 1
        lineCount = 1
        
        
        #print contentsY / fm.height()
        #print contentsY, fm.height(), fm.ascent(), fm.descent()
        
        
        # inefficient code
        def blockiter(start = editor.document().begin()):
            while start.isValid():
                yield start
                start = start.next()
        
        drawn = 0
        for lineCount, block in enumerate(blockiter()):
            lineCount += 1
            
            boundingRect = layout.blockBoundingRect( block )
            position = boundingRect.topLeft()
            if position.y() + boundingRect.height() < contentsY:
                continue
            if position.y() > pageBottom:
                break
            
            txt = QString.number( lineCount )
            p.drawText( self.width() - fm.width(txt), qRound( position.y() ) - contentsY + ascent, txt );
            drawn += 1
        #print "lines drawn: %d" % drawn

class BookmarkColumn(QWidget):
    DEFAULT_STYLE = '''
        border: 2px solid #dea;
    '''
    def __init__(self, parent):
        QWidget.__init__(self, parent)
        self.setMinimumWidth(self.fontMetrics().width('W'))
        self.setStyleSheet(self.DEFAULT_STYLE)
        
        
    def mousePressEvent(self, e):
        editor = self.parent().editor
        
        #layout = editor.document().documentLayout()
        contentsY = editor.verticalScrollBar().value()
        pageBottom = contentsY + editor.viewport().height()
        fm = editor.fontMetrics()
        ascent = editor.fontMetrics().ascent() + 1
        #lineCount = 1
        pprint(locals())


class FoldingColumn(QWidget):
    def __init__(self, parent):
        QWidget.__init__(self, parent)
        self.setMinimumWidth(self.fontMetrics().averageCharWidth() * 1)


class CodeEditor(QWidget):
    
    def __init__(self, parent = None, document = None,):
        '''
        QCodeEditor
        '''
        QWidget.__init__(self)
        self.layout = QHBoxLayout()
        self.layout.setMargin(0)
        self.setLayout(self.layout)
        
        self.setup_bookmark_column()
        self.setup_linenumber_column()
        self.setup_folding_column()
        self.setup_editor()
        
        self.__has_selection = False
        self.connect(self.editor, SIGNAL('copyAvailable(bool)'), 
                                            self.change_has_selection)
        
    def setup_editor(self):
        self.__editor = InternalEditor(self)
        self.layout.addWidget(self.editor)
        #self.editor = InternalEditor(self)
        
    def editor():
        doc = "Editor"
        def fget(self):
            return self.__editor
        return locals()
    editor = property(**editor())
    
    def setup_bookmark_column(self):
        self.bookmark_col = BookmarkColumn(self)
        self.layout.addWidget(self.bookmark_col)
        
    
    def setup_linenumber_column(self):
        self.linenumber_col = LineNumberColumn(self)
        self.layout.addWidget(self.linenumber_col)
    
    def setup_folding_column(self):
        self.folding_col = FoldingColumn(self)
        self.layout.addWidget(self.folding_col)
    
    def focusInEvent(self, e):
        self.editor.setFocus(Qt.TabFocusReason)
        e.accept()
        #QWidget.focusInEvent(self, e)
    
    @pyqtSignature('bool')
    def change_has_selection(self, yes):
        self.__has_selection = yes
        
    def has_selection():
        doc = "Selection available"
        def fget(self):
            return self.__has_selection
        return locals()
    has_selection = property(**has_selection())
        
    def linenumberEnabled():
        doc = "Line numbers column enabled"
        def fget(self):
            return self.linenumber_col.isVisible()
        
        def fset(self, val):
            if val:
                self.linenumber_col.show()
            else:
                self.linenumber_col.hide()
        return locals()
    linenumberEnabled = property(**linenumberEnabled())
    
    
    def bookmarkEnabled():
        doc = "Bookmark column enabled"
        def fget(self):
            return self.bookmark_col.isVisible()
        
        def fset(self, val):
            if val:
                self.bookmark_col.show()
            else:
                self.bookmark_col.hide()
        return locals()
    bookmarkEnabled = property(**bookmarkEnabled())
        
    
    def foldingEnabled():
        doc = "Folding column enabled"
        def fget(self):
            return self.folding_col.isVisible()
        def fset(self, val):
            if val:
                self.folding_col.show()
            else:
                self.folding_col.hide()
        return locals()
    foldingEnabled = property(**foldingEnabled())
    
    def zoomIn(self, range_ = 1):
        '''
        Zooms out 
        '''
        self.editor.zoomIn(range_)
    
    def zoomOut(self, range_ = 1):
        self.editor.zoomOut(range_)
    
    def requestClose(self):
        if self.editor.document().isModified():
            resp = QMessageBox.question(self.parent(), "Close %s?" % self.editor.title, 
                    "Save %s before closing?" % self.editor.title,
                    QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
            if resp == QMessageBox.Cancel:
                return False
            elif resp == QMessageBox.Yes:
                print "Save()"
        return True
        
class CodeEditorFactory(QObject):
    '''
    Handles the document/editor creation
    '''
    def __init__(self, parent):
        QObject.__init__(self, parent)
        opened = {}
        
    def get_editor(self, qurl):
        print "Opening %s" % qurl
        tabPane = self.parent()
        newEditor = tabPane.insert_new()
        newEditor.editor.document().load(qurl)
        
        scheme = qurl.scheme()
        if scheme == 'file':
            path = qurl.path() 
            if QFile.exists(path):
                newEditor = tabPane.insert_new()
                print "Loading document"
                newEditor.editor.document().load(path)
                print "Document loaded"
                #code_editor = CodeEditor( tabPane )
                #print "Creating document"
                #doc = CodeDocument( code_editor.editor, path )
                #print "Doc created"
                
                #print "Vamos a insertar a ", code_editor, code_editor.editor
                #print code_editor.editor.setDocument
                #code_editor.editor.setDocument( doc )
                #print "Doc set"
                #print tabPane
                #print ( code_editor, code_editor.title )
                #tabPane.addWidget( code_editor, code_editor.title )
                ##tabPane.add
        
